#include "beacon.h"
#include "ui.h"      // For drawing, state access, OLED
#include "utils.h"   // For random MAC, etc.
#include "config.h"  // <<< Ensures MAX_LINES_PER_SCREEN is visible
#include "scanner.h" // <<< Included via beacon.h, ensures MAX_APS is visible
#include <U8g2lib.h> // For OLED drawing
#include <cstring>  // For memcpy, strlen
#include <cstddef>  // For offsetof
#include <algorithm> // For std::min

// --- Global Variable Definitions ---
bool beacon_flood_active = false;
uint8_t beacon_target_ap_bssid[6] = {0};
char beacon_target_ap_ssid[33] = {0};
uint8_t beacon_target_ap_channel = 0;

// --- Internal State ---
static BeaconStateType currentBeaconState = BeaconStateType::MODE_SELECTION; // Start at mode selection
static unsigned long last_beacon_time = 0;
static unsigned long last_redraw_time = 0; // <<< Add timer for redraw
static unsigned long last_channel_hop_time = 0;
static uint8_t current_beacon_channel = 1;
static uint32_t beacon_packets_sent = 0;
static int current_ssid_index = 0;
static int selected_mode_index = 0; // 0 = Random, 1 = Target
static int target_ssid_variation_index = 0; // Index for cycling through variations
static const char* mode_names[] = {"Random", "Target"};

// Target AP Selection state
static int selected_ap_index = 0; // Index within the *scanner's* ap_list
static int ap_display_offset = 0; // For scrolling AP list

// Access scanner data (ensure scanner.h included and these are accessible)
extern AccessPointInfo ap_list[MAX_APS];
extern uint8_t ap_count;

// --- SSID List for Flooding ---
// Example list - can be expanded or loaded from SD card later
static const char* beacon_ssid_list[MAX_BEACON_SSIDS] = {
    "Free Public WiFi", "Airport WiFi", "Coffee Shop Guest", "Hotel Guest",
    "FBI Surveillance Van", "Totally Legit Network", "Virus Update Server",
    "Your AP Sucks", "Upgrade Now!", "Network Not Found", "Loading...",
    "Hidden Network", "Pretty Fly for a Wi-Fi", "Get Off My LAN",
    "Definitely Not a Trap", "Secure Network", "Guest Access Point",
    "Mobile Hotspot", "Connect Here", "Ozdag Attacker"
};
static const int beacon_ssid_list_count = sizeof(beacon_ssid_list) / sizeof(char*);

// --- Target Mode SSID Variations ---
static const char* target_suffixes[] = {
    "",        // Original SSID
    ".",
    "..",
    " guest",
    " Setup",
    "_EXT",
    "-5G",      // Common variations
    " Free WiFi"
};
static const int num_target_suffixes = sizeof(target_suffixes) / sizeof(target_suffixes[0]);

// --- External UI Elements ---
extern U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled;
extern bool needsRedraw;
extern UIState currentState; // To return to main menu

// --- Constants (Defined in beacon.h) ---
// #define BEACON_PACKET_INTERVAL 10
// #define BEACON_CHANNEL_HOP_INTERVAL 500 // <<< REMOVE duplicate definition
// #define MAX_BEACON_SSIDS 20
// #define MAX_SSID_LEN 32

// --- Beacon Packet Structure ---
// Based on common beacon frame fields
typedef struct {
    frame_control_t frame_control; // Type: Mgmt, Subtype: Beacon (0x80)
    uint16_t duration_id;          // Typically 0
    uint8_t addr1[6];              // Destination: Broadcast (FF:FF:FF:FF:FF:FF)
    uint8_t addr2[6];              // Source: Random BSSID
    uint8_t addr3[6];              // BSSID: Same as Source
    uint16_t seq_ctrl;             // Sequence number (auto-incremented by SDK if sys_seq=true?)
    // --- Wireless Management Fields ---
    uint64_t timestamp;            // Typically 0 or current time (can be fixed)
    uint16_t beacon_interval;      // Time between beacons (e.g., 100 TU = 102.4ms)
    uint16_t capability_info;      // Capabilities (e.g., 0x0411 for ESS, Privacy, Short Preamble)
    // --- Tagged Parameters ---
    // Tag: SSID parameter set (0)
    uint8_t ssid_tag_num;          // 0
    uint8_t ssid_len;              // Length of SSID
    uint8_t ssid[MAX_SSID_LEN];    // SSID itself
    // Tag: Supported Rates (1)
    uint8_t rates_tag_num;         // 1
    uint8_t rates_len;             // Number of rates (e.g., 8)
    uint8_t rates[8];              // Example: 1, 2, 5.5, 11, 6, 9, 12, 18 Mbps (basic rates marked with 0x80 bit)
    // Tag: DS Parameter set (3)
    uint8_t ds_tag_num;            // 3
    uint8_t ds_len;                // 1
    uint8_t ds_channel;            // Current channel
    // Optional: Add more tags like TIM, Country, RSN if needed
} __attribute__((packed)) beacon_packet_t; // Use packed attribute for precise layout

// Define standard supported rates (802.11b/g) - Basic rates have MSB set
static const uint8_t default_rates[] = {0x82, 0x84, 0x8B, 0x96, 0x0C, 0x12, 0x18, 0x24}; // 1, 2, 5.5, 11, 6, 9, 12, 18 Mbps
static const uint8_t default_rates_len = sizeof(default_rates);

static beacon_packet_t beacon_packet; // Reusable packet buffer

// --- Initialization ---
void beacon_init() {
    Serial.println("Beacon Module Initialized.");
    currentBeaconState = BeaconStateType::MODE_SELECTION; // Start at mode selection
    beacon_flood_active = false;
    current_beacon_channel = 1;
    last_beacon_time = 0;
    last_channel_hop_time = 0;
    beacon_packets_sent = 0;
    current_ssid_index = 0;
    selected_mode_index = 0;
    selected_ap_index = 0;
    ap_display_offset = 0;
    memset(beacon_target_ap_bssid, 0, 6);
    memset(beacon_target_ap_ssid, 0, 33);
    beacon_target_ap_channel = 0;

    // Ensure WiFi is in a state suitable for injection (promiscuous STA)
    // This might already be set by deauth_init, but good to ensure
    wifi_promiscuous_enable(0); // Disable first to reset if needed
    wifi_set_opmode(STATION_MODE);
    // wifi_promiscuous_enable(1); // <<< COMMENT THIS OUT FOR NOW
    // Serial.println("Beacon Init: Promiscuous mode enabled."); // <<< COMMENT THIS OUT

    // Set initial channel (will be overridden if target mode selected)
    // Don't set channel here, wait until flooding starts
    // wifi_set_channel(current_beacon_channel);
    // Serial.printf("Beacon Init: Set initial channel to %d\n", current_beacon_channel);

    needsRedraw = true; // Trigger UI redraw for the beacon screen
}

// --- Stop Function ---
void beacon_stop() {
    if (beacon_flood_active) {
        beacon_flood_active = false;
        currentBeaconState = BeaconStateType::MODE_SELECTION; // Return to mode selection
        Serial.println("Beacon Flood Stopped.");
        // Optional: Restore original channel? Disable promiscuous?
        wifi_promiscuous_enable(0); // <<< Disable promiscuous mode on stop
        Serial.println("Beacon Stop: Disabled promiscuous mode.");
        needsRedraw = true;
    }
}

// --- Main Update Loop ---
void beacon_update() {
    // Only run flooding logic if in a flooding state and active
    if (!beacon_flood_active || (currentBeaconState != BeaconStateType::FLOODING_RANDOM && currentBeaconState != BeaconStateType::FLOODING_TARGET)) {
        return; // Only run logic when actively flooding
    }

    unsigned long current_time = millis();
    bool packet_sent_this_cycle = false; // Flag to check if we sent a packet

    // --- Channel Hopping (Only for Random Mode) ---
    if (currentBeaconState == BeaconStateType::FLOODING_RANDOM) {
        if (current_time - last_channel_hop_time > BEACON_CHANNEL_HOP_INTERVAL) {
            current_beacon_channel = (current_beacon_channel % 14) + 1; // Cycle 1-14
            bool channel_set_ok = wifi_set_channel(current_beacon_channel);
            last_channel_hop_time = current_time;
            // Only log if failed, to reduce spam
            if (!channel_set_ok) {
                Serial.printf("Beacon Random: Set channel %d -> FAIL\n", current_beacon_channel);
            }
        }
    }
    // Note: For FLOODING_TARGET, the channel is fixed and set when starting the flood.

    // --- Send Beacon Packet ---
    if (current_time - last_beacon_time > BEACON_PACKET_INTERVAL) {
        // 1. Select SSID based on mode
        // Use a temporary buffer for the SSID to send
        char ssid_to_send[MAX_SSID_LEN + 1] = {0};
        uint8_t ssid_len_to_send = 0;

        if (currentBeaconState == BeaconStateType::FLOODING_RANDOM) {
            // Use SSID from list
            strncpy(ssid_to_send, beacon_ssid_list[current_ssid_index], MAX_SSID_LEN);
            ssid_len_to_send = strlen(ssid_to_send);
            // Cycle to next list SSID for next time
            current_ssid_index = (current_ssid_index + 1) % beacon_ssid_list_count;
        } else { // FLOODING_TARGET
            // --- Generate SSID Variation for Target Mode ---
            const char* base_ssid = beacon_target_ap_ssid;
            const char* suffix = target_suffixes[target_ssid_variation_index];
            size_t base_len = strlen(base_ssid);
            size_t suffix_len = strlen(suffix);

            // Construct the SSID, ensuring it fits MAX_SSID_LEN
            if (base_len + suffix_len <= MAX_SSID_LEN) {
                snprintf(ssid_to_send, sizeof(ssid_to_send), "%s%s", base_ssid, suffix);
            } else {
                // Suffix doesn't fit, just use the base SSID (truncated if necessary)
                strncpy(ssid_to_send, base_ssid, MAX_SSID_LEN);
            }
            ssid_len_to_send = strlen(ssid_to_send); // Get length of the final SSID

            target_ssid_variation_index = (target_ssid_variation_index + 1) % num_target_suffixes; // Cycle to next suffix for the next packet
        }

        // 2. Generate random BSSID (Source MAC) - Use random for both modes for now
        uint8_t random_mac[6];
        utils_random_mac(random_mac); // Use utility function

        // 3. Fill beacon_packet structure
        // Frame Control: Type Mgmt (00), Subtype Beacon (1000) -> 0x80
        beacon_packet.frame_control.type = 0;    // Management
        beacon_packet.frame_control.subtype = 8; // Beacon
        beacon_packet.frame_control.flags = 0x00; // Set all flags (ToDS, FromDS, MoreFrag, Retry, PwrMgmt, MoreData, Protected, Order) to 0 for open network

        beacon_packet.duration_id = 0;
        memset(beacon_packet.addr1, 0xFF, 6); // Destination: Broadcast
        memcpy(beacon_packet.addr2, random_mac, 6); // Source: Random MAC
        memcpy(beacon_packet.addr3, random_mac, 6); // BSSID: Random MAC
        beacon_packet.seq_ctrl = 0; // Let SDK handle sequence number (sys_seq=true)

        beacon_packet.timestamp = 0; // Can be micros() but 0 is fine
        beacon_packet.beacon_interval = 100;    // 100 TU = 102.4 ms
        beacon_packet.capability_info = 0x0411; // ESS, Short Preamble, Short Slot Time (Open Network)
        // Use 0x0431 if you want to set the Protected Frame bit in flags to 1 (beacon_packet.frame_control.flags = 0x40;)

        // Tagged Parameters
        beacon_packet.ssid_tag_num = 0; // Tag 0: SSID
        beacon_packet.ssid_len = ssid_len_to_send;
        memcpy(beacon_packet.ssid, ssid_to_send, ssid_len_to_send);

        beacon_packet.rates_tag_num = 1; // Tag 1: Supported Rates
        beacon_packet.rates_len = default_rates_len;
        memcpy(beacon_packet.rates, default_rates, default_rates_len);

        beacon_packet.ds_tag_num = 3; // Tag 3: DS Parameter Set (Channel)
        beacon_packet.ds_len = 1;
        beacon_packet.ds_channel = current_beacon_channel;

        // 4. Calculate actual packet size
        // Correct calculation: Size up to SSID field + Tag overhead + Actual SSID Len + Rates Tag overhead + Rates Len + DS Tag overhead + DS Channel Len
        size_t packet_size = offsetof(beacon_packet_t, ssid) // Size up to SSID field start
                             + 2 // SSID Tag Num (1) + SSID Len (1)
                             + ssid_len_to_send // Actual SSID length
                             + 2 // Rates Tag Num (1) + Rates Len (1)
                             + default_rates_len // Actual rates length
                             + 2 // DS Tag Num (1) + DS Len (1)
                             + 1; // DS Channel (1 byte)

        // 4. Call wifi_send_pkt_freedom
        int result = wifi_send_pkt_freedom((uint8_t*)&beacon_packet, packet_size, false); // sys_seq=false for now

        // <<< Log EVERY send attempt and result >>>
        Serial.printf("Beacon Send: Ch=%d, SSID='%s', Size=%d, Result=%d\n",
                      current_beacon_channel, ssid_to_send, packet_size, result);

        if (result == 0) { // Check if sending was successful
            beacon_packets_sent++; // Increment only on success
            packet_sent_this_cycle = true; // Mark that we sent one
        } else {
            Serial.printf("!!! Beacon send failed! Result: %d\n", result); // <<< Log failures
        }

        last_beacon_time = current_time;
    }

    // --- Trigger Redraw Periodically ---
    // Update roughly twice per second if packets were sent
    if (packet_sent_this_cycle && (current_time - last_redraw_time > 500)) {
        needsRedraw = true;
        last_redraw_time = current_time;
    }

    // Yield occasionally to prevent WDT resets
    yield();
}

// --- UI Drawing Function ---
// Helper to draw AP list for selection
void beacon_draw_target_selection() {
    oled.setFont(u8g2_font_5x7_tf); // Use smaller font for title
    int y = 22; // Start position below header
    oled.drawStr(0, y, "Select Target AP:");
    y += 10; // Move down for the list

    oled.setFont(u8g2_font_5x7_tf); // Use smaller font for list items
    int line_height = 9; // Adjusted line height for 5x7 font
    const int items_per_page = 3; // Show 3 items

    // Adjust offset if selection goes off screen
    if (selected_ap_index < ap_display_offset) {
        ap_display_offset = selected_ap_index;
    }
    // Ensure offset doesn't make selection go past the last item on the screen
    if (selected_ap_index >= ap_display_offset + items_per_page) {
        ap_display_offset = selected_ap_index - items_per_page + 1;
    }
    // Ensure offset doesn't go beyond possible start indices
    if (ap_count > items_per_page && ap_display_offset > ap_count - items_per_page) {
        ap_display_offset = ap_count - items_per_page;
    }
    if (ap_display_offset < 0) { // Prevent negative offset
        ap_display_offset = 0;
    }

    for (int i = 0; i < items_per_page; ++i) { // Loop only 3 times
        int current_ap_list_index = ap_display_offset + i;
        if (current_ap_list_index >= ap_count) break; // Don't draw past the end

        AccessPointInfo* ap = &ap_list[current_ap_list_index];
        char line_buffer[40]; // Buffer for SSID + channel

        // Indicate selection
        char indicator = (current_ap_list_index == selected_ap_index) ? '>' : ' ';

        // Format SSID (truncate if needed) and channel
        snprintf(line_buffer, sizeof(line_buffer), "%c%.16s (%d)",
                 indicator,
                 ap->ssid[0] == '\0' ? "[Hidden]" : ap->ssid, // Handle hidden SSIDs
                 ap->channel);

        oled.drawStr(0, y + i * line_height, line_buffer); // Use adjusted line_height
    }

    if (ap_count == 0) {
        oled.setFont(u8g2_font_6x10_tf); // Switch back for message
        oled.drawStr(10, 40, "No APs found!");
        oled.drawStr(10, 55, "Run Scanner first.");
    }
    
    // Draw scroll indicators if necessary
    if (ap_count > items_per_page) {
        if (ap_display_offset > 0) {
            oled.drawTriangle(SCREEN_WIDTH - 6, 16, SCREEN_WIDTH - 9, 20, SCREEN_WIDTH - 3, 20); // Up arrow (position adjusted)
        }
        if (ap_display_offset + items_per_page < ap_count) {
             oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 3, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 7, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 7); // Down arrow
        }
    }
    
    oled.setFont(u8g2_font_5x7_tf); // Ensure smaller font for footer
    oled.drawStr(0, SCREEN_HEIGHT -1, "SHORT:Next LONG:Select");
    // Reset font if needed before returning (good practice)
    oled.setFont(u8g2_font_6x10_tf); 
}

void beacon_draw() {
    // Draw UI based on currentBeaconState
    oled.setFont(u8g2_font_6x10_tf); // Consistent font

    switch (currentBeaconState) {
    case BeaconStateType::MODE_SELECTION:
        oled.drawStr(0, 25, "Beacon Flood Mode:");
        for (int i = 0; i < 2; ++i) {
            char indicator = (i == selected_mode_index) ? '>' : ' ';
            oled.drawStr(10, 40 + i * 12, String(indicator + String(mode_names[i])).c_str());
        }
         oled.setFont(u8g2_font_5x7_tf); // Use smaller font for footer
         oled.drawStr(0, SCREEN_HEIGHT -1, "SHORT:Next LONG:Select");
         oled.setFont(u8g2_font_6x10_tf); // Reset font if needed for other parts
        break;
    case BeaconStateType::SELECT_TARGET_AP:
        beacon_draw_target_selection(); // Call helper
        break;
    case BeaconStateType::FLOODING_RANDOM:
    case BeaconStateType::FLOODING_TARGET: // Same display for both flooding modes
        oled.drawStr(0, 25, "Beacon Flood Active!");
        char count_str[30];
        snprintf(count_str, sizeof(count_str), "Sent: %lu", beacon_packets_sent);
        oled.drawStr(0, 40, count_str);
        char chan_str[20];
        // Show target SSID/Channel if target mode
        if (currentBeaconState == BeaconStateType::FLOODING_TARGET) {
             snprintf(chan_str, sizeof(chan_str), "Tgt: %.10s Ch:%d", beacon_target_ap_ssid, current_beacon_channel);
        } else {
             snprintf(chan_str, sizeof(chan_str), "Channel: %d (Hop)", current_beacon_channel);
        }
        oled.drawStr(0, 55, chan_str);
        oled.setFont(u8g2_font_5x7_tf); // Use smaller font for footer
        oled.drawStr(0, SCREEN_HEIGHT -1, "LONG: Stop");
        oled.setFont(u8g2_font_6x10_tf); // Reset font
        break;
    }
}

// --- Input Handling Function ---
void beacon_handle_input(ButtonPressType pressType) {
    switch (currentBeaconState) {
    case BeaconStateType::MODE_SELECTION:
        if (pressType == ButtonPressType::SHORT_PRESS) {
            selected_mode_index = (selected_mode_index + 1) % 2;
            needsRedraw = true;
        } else if (pressType == ButtonPressType::LONG_PRESS) {
            if (selected_mode_index == 0) { // Random selected
                // Start Random flooding
                beacon_flood_active = true;
                currentBeaconState = BeaconStateType::FLOODING_RANDOM;
                beacon_packets_sent = 0;
                current_beacon_channel = 1; // Start channel hopping at 1
                wifi_set_channel(current_beacon_channel);
                last_channel_hop_time = millis();
                last_beacon_time = millis();
                current_ssid_index = 0;
                Serial.println("Beacon Flood Started (Random Mode).");
                // <<< Enable Promiscuous Mode >>>
                wifi_promiscuous_enable(1);
                Serial.println("Beacon Start (Random): Enabled promiscuous mode.");
                needsRedraw = true;
            } else { // Target selected
                // Go to AP selection screen
                currentBeaconState = BeaconStateType::SELECT_TARGET_AP;
                selected_ap_index = 0; // Reset selection
                ap_display_offset = 0;
                needsRedraw = true;
            }
        }
        break;

    case BeaconStateType::SELECT_TARGET_AP:
        if (ap_count == 0) { // No APs to select
             if (pressType == ButtonPressType::LONG_PRESS) { // Go back
                 currentBeaconState = BeaconStateType::MODE_SELECTION;
                 needsRedraw = true;
             }
             break;
        }
        if (pressType == ButtonPressType::SHORT_PRESS) {
            selected_ap_index = (selected_ap_index + 1) % ap_count;
            needsRedraw = true;
        } else if (pressType == ButtonPressType::LONG_PRESS) {
            // Target AP selected, store its info
            AccessPointInfo* target_ap = &ap_list[selected_ap_index];
            memcpy(beacon_target_ap_bssid, target_ap->bssid, 6);
            strncpy(beacon_target_ap_ssid, target_ap->ssid, 32);
            beacon_target_ap_ssid[32] = '\0'; // Ensure null termination
            beacon_target_ap_channel = target_ap->channel;

            // Start Target flooding
            beacon_flood_active = true;
            currentBeaconState = BeaconStateType::FLOODING_TARGET;
            beacon_packets_sent = 0;
            current_beacon_channel = beacon_target_ap_channel; // Use target channel
            Serial.printf("Beacon Start: Setting fixed channel %d for target '%s'\n", current_beacon_channel, beacon_target_ap_ssid);
            wifi_set_channel(current_beacon_channel);
            last_beacon_time = millis();
            // No channel hopping, so last_channel_hop_time isn't used here
            // <<< Enable Promiscuous Mode >>>
            wifi_promiscuous_enable(1);
            Serial.println("Beacon Start (Target): Enabled promiscuous mode.");
            needsRedraw = true;
            Serial.println("Beacon Flood Started (Target Mode).");
        }
        break;

    case BeaconStateType::FLOODING_RANDOM:
    case BeaconStateType::FLOODING_TARGET:
        if (pressType == ButtonPressType::LONG_PRESS) {
            beacon_stop(); // Use the stop function
        }
        break;
    }
} 